@@ -1,5 +1,5 @@
 from collections import deque
-from typing import Any, Deque, List, Tuple
+from typing import Any
 
 from aiosmtplib.response import SMTPResponse
 from aiosmtplib.smtp import SMTP
@@ -11,8 +11,8 @@ class DummySMTPAuth(SMTP):
     def __init__(self, *args: Any, **kwargs: Any):
         super().__init__(*args, **kwargs)
 
-        self.received_commands: List[bytes] = []
-        self.responses: Deque[Tuple[int, str]] = deque()
+        self.received_commands: list[bytes] = []
+        self.responses: deque[tuple[int, str]] = deque()
         self.esmtp_extensions = {"auth": ""}
         self.server_auth_methods = ["cram-md5", "login", "plain"]
         self.supports_esmtp = True
