@@ -1,11 +1,6 @@
 import asyncio
 import sys
 
-import pytest
-
-
-pytestmark = pytest.mark.asyncio()
-
 
 async def test_command_line_send(hostname: str, smtpd_server_port: int) -> None:
     proc = await asyncio.create_subprocess_exec(
