@@ -5,12 +5,12 @@ from aiosmtplib.response import SMTPResponse
 
 
 @given(integers(), text())
-def test_response_repr(code, message):
+def test_response_repr(code: int, message: str) -> None:
     response = SMTPResponse(code, message)
-    assert repr(response) == "({}, {})".format(response.code, response.message)
+    assert repr(response) == f"({response.code}, {response.message})"
 
 
 @given(integers(), text())
-def test_response_str(code, message):
+def test_response_str(code: int, message: str) -> None:
     response = SMTPResponse(code, message)
-    assert str(response) == "{} {}".format(response.code, response.message)
+    assert str(response) == f"{response.code} {response.message}"
